package com.zyzf.coffeepreorder.composeui

import android.content.res.Configuration
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.heightIn
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Add
import androidx.compose.material.icons.filled.Clear
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExtendedFloatingActionButton
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.navigation.NavController
import coil.compose.AsyncImage
import coil.request.ImageRequest
import com.zyzf.coffeepreorder.R
import com.zyzf.coffeepreorder.composeui.navigation.Screen
import com.zyzf.coffeepreorder.database.AppDatabase
import com.zyzf.coffeepreorder.database.model.Coffee
import com.zyzf.coffeepreorder.ui.theme.CoffeePreorderTheme
import kotlinx.coroutines.DelicateCoroutinesApi
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

@OptIn(ExperimentalMaterial3Api::class, DelicateCoroutinesApi::class)
@Composable
fun Cart(navController: NavController?) {
    val openDialog = remember { mutableStateOf(false) }
    var coffee by remember { mutableStateOf(Coffee("", 0.0, "", null, 0)) }
    val context = LocalContext.current
    val itemsList = remember { mutableStateListOf<Coffee>() }
    LaunchedEffect(Unit) {
        withContext(Dispatchers.IO) {
            AppDatabase.getInstance(context).coffeeDao().getAllInCart().collect { data ->
                itemsList.clear()
                itemsList.addAll(data)
            }
        }
    }
    LazyColumn(
        horizontalAlignment = Alignment.CenterHorizontally) {
        items(itemsList) { currentCoffee ->
            Row(modifier = Modifier
                .fillMaxWidth()
                .heightIn(max = 140.dp)
                .padding(bottom = 10.dp, top = 10.dp),
                horizontalArrangement = Arrangement.SpaceAround) {
                AsyncImage(
                    model = ImageRequest.Builder(context = LocalContext.current).data("https://zyzf.space/s/zXgFRTmbR4KMxMH/download?path=&files=coffee_image_" + currentCoffee.uid +".png")
                        .crossfade(true).build(),
                    error = painterResource(R.drawable.ic_broken_image),
                    placeholder = painterResource(R.drawable.loading_img),
                    contentDescription = "Кофе",
                    contentScale = ContentScale.Crop,
                    modifier = Modifier
                        .size(100.dp)
                        .clip(RoundedCornerShape(50.dp))
                )
                Column(
                    Modifier
                        .weight(2f)
                        .padding(start = 20.dp)) {
                    val currentCoffeeName: String = if (currentCoffee.count > 1) {
                        currentCoffee.name + " x" + currentCoffee.count.toString()
                    } else {
                        currentCoffee.name
                    }
                    Text(text = currentCoffeeName, fontSize = 25.sp)
                    Text(text = String.format("%.2f", currentCoffee.cost), fontSize = 20.sp)
                    Text(text = currentCoffee.ingredients, fontSize = 15.sp)
                    Row(
                        Modifier
                            .fillMaxWidth()
                            .padding(top = 5.dp)) {
                        Button(
                            onClick = { coffee = currentCoffee; openDialog.value = true },
                            shape = CircleShape,
                            modifier = Modifier.padding(start = 10.dp, end = 10.dp),
                            colors = ButtonDefaults.buttonColors(
                                containerColor = MaterialTheme.colorScheme.errorContainer,
                                contentColor = MaterialTheme.colorScheme.error
                            )
                        ) {
                            Icon(
                                imageVector = Icons.Default.Clear,
                                contentDescription = "Favorite",
                                modifier = Modifier.size(20.dp)
                            )
                            Spacer(Modifier.size(ButtonDefaults.IconSpacing))
                            Text(text = "Удалить из корзины")
                        }
                    }
                }
            }
        }
    }
    Box(modifier = Modifier.fillMaxSize()) {
        ExtendedFloatingActionButton(
            onClick = { navController?.navigate(Screen.Order.route) },
            icon = { Icon(Icons.Filled.Add, "Сделать заказ") },
            text = { Text(text = "Сделать заказ") },
            modifier = Modifier.align(alignment = Alignment.BottomEnd).padding(all = 20.dp),
            containerColor = MaterialTheme.colorScheme.primaryContainer,
            contentColor = MaterialTheme.colorScheme.primary
        )
    }
    if (openDialog.value) {
        AlertDialog(
            icon = {
                Icon(Icons.Default.Clear, contentDescription = "Example Icon")
            },
            title = {
                Text(text = "Подтверждение")
            },
            text = {
                Text(text = "Вы хотите удалить элемент из корзины?")
            },
            onDismissRequest = {
                openDialog.value = false
            },
            confirmButton = {
                TextButton(
                    onClick = {
                        GlobalScope.launch (Dispatchers.Main) {
                            coffee.uid?.let {
                                AppDatabase.getInstance(context).cartDao().deleteCoffee(it, 1)
                            }
                        }

                        openDialog.value = false
                    }
                ) {
                    Text("Да")
                }
            },
            dismissButton = {
                TextButton(
                    onClick = {
                        openDialog.value = false
                    }
                ) {
                    Text("Нет")
                }
            }
        )
    }
}

@Preview(name = "Light Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_NO)
@Preview(name = "Dark Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_YES)
@Composable
fun CoffeeListPreview() {
    CoffeePreorderTheme {
        Surface(
            color = MaterialTheme.colorScheme.background
        ) {
            Cart(navController = null)
        }
    }
}